@@ -1,5 +1,5 @@
-#ifndef LINC_H
-#define LINC_H 1
+#ifndef _LINC_H_
+#define _LINC_H_
 
 #include <linc/linc-config.h>
 #include <linc/linc-types.h>
@@ -7,9 +7,11 @@
 #include <linc/linc-connection.h>
 #include <linc/linc-server.h>
 
+G_BEGIN_DECLS
+
 extern GMainLoop *linc_loop;
 
-void       linc_init            (void);
+void       linc_init            (gboolean       init_threads);
 
 LincWatch *linc_io_add_watch    (GIOChannel    *channel,
 				 GIOCondition   condition,
@@ -20,4 +22,6 @@ void       linc_main_iteration  (gboolean       block_for_reply);
 gboolean   linc_main_pending    (void);
 void       linc_main_loop_run   (void);
 
-#endif
+G_END_DECLS
+
+#endif /* _LINC_H_ */
